def get_landscape(region):
    """Takes the region and returns the correct landscape name

    Args:
        region (str): a AWS region name, such as 'eu-west-1'

    Returns:
        String

    """
    return {
        'us-east-1': 'na',
        'eu-central-1': 'eu',
        'eu-west-1': 'uk'
    }.get(region, 'eu')

def get_environment(env):
    return {
        'prod': 'production',
        'preprod': 'preprod',
        'production': 'production'
    }.get(env, 'preprod')



'''
ECS Auditor class to get the current image and param version for all ECS Services
'''
import json
from collections import OrderedDict
from jsondiff import diff
import ecs_auditor.config as config
from ecs_auditor.helpers.github_helper import GithubHelper
from ecs_auditor.helpers.ecs_helper import EcsHelper
from ecs_auditor.helpers.ssm_helper import SsmHelper
from ecs_auditor.helpers.excel_helper import ExcelHelper
from ecs_auditor.lib import base

class Client(object):
    """docstring for Main.
    """

    def __init__(self, config_file_path, region, output, env, services, ignore_author, show_differences_only):
        """Initializer

        Args:
            region (str):   AWS Region
            output (str):   Preferred output style
            env (str):      ECS Cluster environment

        Returns:
            N/A

        """
        super(Client, self).__init__()
        self.config_file_path = config_file_path
        self.region = region
        self.output = output
        self.env = env
        self.ignore_author = ignore_author
        self.services = services
        self.diff_mode = show_differences_only

    def perform(self):
        for cluster in config.settings['clusters']:
            if (self.region != 'all' and self.region != cluster['region']) or (self.env != 'all' and self.env != cluster['env']):
                continue

            print "*--------------------------------*"
            print "Getting data for {} {}".format(cluster['region'], cluster['env'])
            print "*--------------------------------*"

            ecs = EcsHelper(cluster['region'])
            ssm = SsmHelper(cluster['region'])
            cluster['services'] = []
            services = []
            task_definitions = []
            service_list_arns = ecs.list_services(cluster['name'], [], '')

            for service_chunk in [service_list_arns[i:i + 10] for i in xrange(0, len(service_list_arns), 10)]:
                for service in ecs.client.describe_services(cluster=cluster['name'], services=service_chunk)['services']:
                    name = "_".join((service['serviceName'].split('-')[3:-4]))
                    if self.services is not '' and name in self.services.split(','):
                        services.append(service)
                    elif self.services is '':
                        services.append(service)

            for service in services:
                task_definitions.append(ecs.client.describe_task_definition(
                    taskDefinition=service['taskDefinition']
                ))

            for task_definition in task_definitions:
                dict_ordered = None
                for containerDefinition in task_definition['taskDefinition']['containerDefinitions']:
                    for param in containerDefinition['environment']:
                        if param['name'] == "VERSION" or param['name'] == "PARAM_VERSION":
                            dict_ordered = None
                            image_tag = containerDefinition['image'].split(':')[-1]
                            regionEnv = "{}.{}".format(base.get_landscape(cluster['region']), base.get_environment(cluster['env']))
                            print containerDefinition['name']

                            author = ssm.get_author(regionEnv, containerDefinition['name'], param['value'])

                            try:
                                author = ("".join(author).split('/')[-1:])[0]
                            except Exception as e:
                                author = None
                            github = GithubHelper().get_param_version_data(containerDefinition['name'], base.get_landscape(cluster['region']), cluster['env'])
                            version = regionEnv + "." + containerDefinition['name'].replace('-', '_') + "." + param['value']
                            ssm_version = ssm.get_param(version)

                            diff_output = diff(ssm_version, github['params'])

                            if self.diff_mode is True and author != self.ignore_author and (len(diff_output) > 0 or param['value'] != github['version'] or author != 'codeship_param_pusher'):
                                dict_ordered = OrderedDict([('name', containerDefinition['name']), ('region', regionEnv), ('image_tag', image_tag), ('running_param_version', param['value']), ('github_param_version', github['version']), ('author', author), ('diff', diff_output)])
                                break
                            elif self.diff_mode is False and author != self.ignore_author:
                                dict_ordered = OrderedDict([('name', containerDefinition['name']), ('region', regionEnv), ('image_tag', image_tag), ('running_param_version', param['value']), ('github_param_version', github['version']), ('author', author), ('diff', diff_output)])
                                break

                if dict_ordered is not None: cluster['services'].append(dict_ordered)

            print "\n\n"

        self.output_data()

    def output_data(self):
        if self.output == 'excel':
            ExcelHelper().create(config.settings['clusters'])
            return None

        for cluster in config.settings['clusters']:
            if 'services' not in cluster: continue

            print "*--------------------------------*"
            print "Output for {} {}".format(cluster['region'], cluster['env'])
            print "*--------------------------------*"

            if self.output == "json":
                print json.dumps(cluster['services'], indent=4)
            elif self.output == "pretty":
                for service in cluster['services']:
                    print service['name']
                    print "---> PARAM_VERSION: {}, IMAGE_TAG: {}, AUTHOR: {}".format(service['running_param_version'], service['image_tag'], service['author'])
                    print "---> GITHUB_VERSION: {}".format(service['github_param_version'])
                    print "---> DIFF"
                    print service['diff']
                    print "\n"
            elif self.output == "compare":
                compare = {}
                for service in cluster['services']:
                    if self.ignore_author not in service['author']:
                        regionEnv = "{}-{}".format(base.get_landscape(cluster['region']), cluster['env'])
                        if service['name'] not in compare:
                            compare["{}".format(service['name'])] = {}
                        compare[service['name']][regionEnv] = {'image_tag': service['image_tag'], 'running_param_version': service['param_version'], 'author': service['author']}
                print json.dumps(compare, indent=4, sort_keys=True)



import argparse
import ecs_auditor
from ecs_auditor.client import Client
import ecs_auditor.config as config

def execute():
    """Entry point for the application script"""
    parser = argparse.ArgumentParser(description='Command line arguments')
    parser.add_argument('--region', metavar='[region]', default='all', help='AWS region to get data for? [default: all, options: [us-east-1, eu-central-1, eu-west-1, all] ]')
    parser.add_argument('--env', metavar='[env]', default='all', help='AWS environment to get data for? [default: all, options: [preprod, prod, all] ]')
    parser.add_argument('--output', metavar='[output]', default='pretty', help='How to display the results. [default: pretty, options: [pretty, json, compare] ]')
    parser.add_argument('--ignore-author', metavar='[ignore-author]', default='', help='Results exludung specific param author. [default: None, options: string')
    parser.add_argument('--services', metavar='[services]', default='', help='Specify a single service. [default: None, options: string')
    parser.add_argument('--show-differences-only', default=False, type=lambda x: (str(x).lower() == 'true'))
    parser.add_argument('--config', help='Specify location of the config file')
    args = parser.parse_args()
    config.settings = config.load_config(args.config)

    Client(args.config, args.region.lower(), args.output.lower(), args.env.lower(), args.services.lower(), args.ignore_author.lower(), args.show_differences_only).perform()

if __name__ == '__main__':
    execute()



'''
Excep Helper class to generate an excel list for the parameters
'''
import xlsxwriter
import datetime
import operator

class ExcelHelper(object):
    """docstring for ExcelHelper.
    """

    def __init__(self):
        """Initializer

        Args:
            N/A

        Returns:
            N/A

        """
        super(ExcelHelper, self).__init__()

    def create(self, args):
        '''Create an excel file with the data in it

        Args:
            N/A

        Returns:
            list
        '''
        # Create a workbook and add a worksheet.
        workbook = xlsxwriter.Workbook("./data/ecs-auditor-{}.xlsx".format(datetime.datetime.now()))
        worksheet = workbook.add_worksheet()
        title_cell_format = workbook.add_format({'bold': True, 'font_color': 'red', 'font_size': '16'})
        entry_wrapping = workbook.add_format({'align': 'top'})
        cell_wrapping = workbook.add_format({'text_wrap': True, 'align': 'top'})

        row = 1

        for cluster in args:
            if 'services' not in cluster: continue

            print "*--------------------------------*"
            print "Writing to excel for {} {}".format(cluster['region'], cluster['env'])
            print "*--------------------------------*"
            cluster['services'].sort(key=operator.itemgetter('name'))
            for service in cluster['services']:
                col = 0

                for (key, value) in (service).iteritems():
                    # print key on row above
                    worksheet.set_column(col, col, 25)
                    key = key.replace('_', ' ').title()
                    worksheet.write(0, col, key, title_cell_format)

                    # print value row below
                    if isinstance(value, dict):
                        dict_data = ""
                        for i, (j, k) in enumerate((value).iteritems()):
                            dict_data += "{}={}".format(j, k)
                            if i != len(value)-1:
                                dict_data += "\n"
                        worksheet.write(row, col, dict_data, cell_wrapping)
                    else:
                        worksheet.write(row, col, value, entry_wrapping)
                        col += 1
                row += 1

        workbook.close()



'''
Ssm Helper class to provide an interface to retrieve parameter data from AWS SSM
'''
import boto3
import base64, zlib
import simplejson

class SsmHelper(object):
    """docstring for SsmHelper.
    """

    def __init__(self, region):
        """Initializer

        Args:
            region (str)

        Returns:
            N/A

        """
        super(SsmHelper, self).__init__()
        self.region = region
        self.client = boto3.Session(region_name=self.region).client('ssm')

    def get_param(self, name):
        """Retrieve a parameter from SSM

        Args:
            region (str)

        Returns:
            Dict

        """
        try:
            rtn = self.client.get_parameter(Name=name, WithDecryption=True)

            if len(rtn['Parameter']) > 0:
                json_resp = zlib.decompress(base64.b64decode(
                    rtn['Parameter']['Value']), 16 + zlib.MAX_WBITS
                                           ).decode('utf-8')
                return simplejson.loads(json_resp)
        except Exception as e:
            print "Parameter {} not found.. skipping.".format(name)

        return {}

    def get_author(self, prefix, application, version):
        """Get the Author of a SSM parameter push

        Args:
            prefix (str): the prefix of the parameter
            application (str): the application name for the parameter
            version (str): the version of the parameter

        Returns:
            str

        """
        print "{}.{}.{}".format(prefix, application.replace('-', '_'), version)
        filters=[
            {
                'Key': 'Name',
                'Values': [
                    "{}.{}.{}".format(prefix, application.replace('-', '_'), version),
                ],
            }
        ]

        rtn = self.client.describe_parameters(Filters=filters)

        if len(rtn['Parameters']) == 0:
            while rtn.get('NextToken') and (rtn['NextToken'] is not None or rtn['NextToken'] != '') and (len(rtn.get('Parameters')) <= 0):
                rtn = self.client.describe_parameters(
                    Filters=filters,
                    NextToken=rtn['NextToken']
                )

        if len(rtn['Parameters']) > 0:
            return rtn['Parameters'][0]['LastModifiedUser']

        return ""



import os
import sys
import yaml

settings=None

def get_relative_path():
    return os.path.abspath(os.path.dirname(__file__))

def load_config(file_name):
    f = open("{}/{}".format( os.getcwd(),file_name))
    data = yaml.load(f)
    f.close()

    return data



#!/usr/bin/env python
"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/SpekoTechnologies/ecs-auditor
"""

# Always prefer setuptools over distutils
from io import open
from setuptools import setup, find_packages

setup(
    name='ecs-auditor',  # Required
    version='1.0.3',  # Required
    url='https://github.com/SpekoTechnologies/ecs-auditor',  # Optional
    author='Speko Technologies',  # Optional
    author_email='info@speko.io',  # Optional
    description='An ECS Auditor',  # Required
    packages=find_packages(),  # Required
    long_description=open('README.rst').read(),  # Optional
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    setup_requires=[
        "ansible",
    ],
    install_requires=[
        "boto3==1.7.41",
        "jsondiff",
        "PyGithub==1.40",
        "python-coveralls==2.7.0",
        "python-dateutil==2.7.3",
        "python-jose-ext==1.3.2.2",
        "simplejson",
        "pyaml==17.10.0",
        "ansible-vault",
        "xlsxwriter",
    ],
    entry_points={  # Optional
        'console_scripts': [
            'ecs-auditor=ecs_auditor.cmdline:execute',
        ],
    },
    keywords='ecs auditor aws ssm python',  # Optional
)



'''
Github Helper class to provide an interface with Github api
'''
from github import Github as GithubExternal
import base64
import yaml
from ansible_vault import Vault
import ecs_auditor.config as config

class GithubHelper(object):
    """docstring for Github.
    """

    def __init__(self):
        """Initializer

        Args:
            N/A

        Returns:
            N/A

        """
        super(GithubHelper, self).__init__()
        self.client = GithubExternal(config.settings['common']['gh_access_token'])
        self.organization = self.client.get_organization(config.settings['common']['gh_org'])

    def get_param_version_data(self, app_name, landscape, environment):
        """Retrieves the content of the vault and playbook file

        Args:
            app_name (str): the name of the application
            landscape (str): the landscape where the application is
            environment (str): the environment of the application

        Returns:
            String

        """
        data = {}
        rtn = {'version': 'NOT FOUND', 'params': {}}

        repo = self.organization.get_repo(config.settings['common']['gh_repo'])
        file_name = app_name.replace('-', '_')

        file_options = [
            "{}/{}/{}".format(file_name, environment, landscape.upper()),
            "{}/{}/{}".format(file_name, environment, landscape)
        ]

        for option in file_options:
            try:
                files = repo.get_file_contents(option)
                for file_path in files:
                    if "vault" in file_path.path:
                        data['vault'] = repo.get_file_contents(file_path.path).content
                    else:
                        data['content'] = repo.get_file_contents(file_path.path).content
                break
            except Exception as e:
                pass

        if 'content' in data:
            content = yaml.load(base64.b64decode(data['content']))
            rtn['version'] = content[0]['tasks'][1]['param_pusher']['version']
            rtn['params'] = self.combine_vault_with_playbook(data, content)

        return rtn

    def combine_vault_with_playbook(self, data, content):
        """Combines the vault data with the playbook data

        Args:
            data (dict)
            content (dict)

        Returns:
            dict

        """
        vault = Vault(config.settings['common']['vault_enc_key'])
        secrets = vault.load(base64.b64decode(data['vault']))

        for k, v in content[0]['tasks'][1]['param_pusher']['parameters'].iteritems():
            if ('VAULT' in v) and (k in content[0]['tasks'][1]['param_pusher']['parameters']):
                if "VAULT_{}".format(k) in secrets:
                    content[0]['tasks'][1]['param_pusher']['parameters'][k] = secrets["VAULT_" + k]
                else:
                    print v
                    print "VAULT_{}".format(k)
                    print secrets
                    content[0]['tasks'][1]['param_pusher']['parameters'][k] = "KEY NAME MISMATCH"

        return content[0]['tasks'][1]['param_pusher']['parameters']



'''
ECS Helper class to provide interactions with the ECS API
'''
import boto3

class EcsHelper(object):
    """docstring for EcsHelper.
    """

    def __init__(self, region):
        """Initializer

        Args:
            N/A

        Returns:
            N/A

        """
        super(EcsHelper, self).__init__()
        self.region = region
        self.client = boto3.Session(region_name=self.region).client('ecs')

    def list_services(self, cluster, arn_list, token):
        '''Return list of services for an environment/region pair

        Args:
            N/A

        Returns:
            list
        '''
        response = self.client.list_services(
            cluster=cluster,
            nextToken=token
        )

        for service in response['serviceArns']:
            arn_list.append(service)

        if 'nextToken' in response:
            self.list_services(cluster, arn_list, token=response['nextToken'])

        return arn_list
